<?php
namespace Topalis\App\Job;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;
use Topalis\App\Entity\AppStateEntityManager;
use Topalis\App\Entity\IosAppState;
use Topalis\App\Exception\VppAlreadyAssociatedException;
use Topalis\App\Exception\VppException;
use Topalis\App\Ios\VppLicenseService;
use Topalis\Kernel\Job\WorkerInterface;
use Topalis\MDM\MdmAdmin;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class VppAssociateWorker implements WorkerInterface, LoggerAwareInterface
{

    /**
     * @var VppLicenseService
     */
    protected $vppLicenseService;

    /**
     * @var AppStateEntityManager
     */
    protected $appStateEntityManager;

    /**
     * @var MdmAdmin
     */
    protected $mdmAdmin;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    public function __construct(
        VppLicenseService $vppLicenseService,
        AppStateEntityManager $appStateEntityManager,
        MdmAdmin $mdmAdmin
    )
    {
        $this->vppLicenseService = $vppLicenseService;
        $this->appStateEntityManager = $appStateEntityManager;
        $this->mdmAdmin = $mdmAdmin;
    }

    public function handle(array $data)
    {
        /** @var IosAppState $state */
        $state = $this->appStateEntityManager->find($data['appState']);
        if (null === $state) {
            throw new \RuntimeException("No state found with id " . $data['appState']);
        }
        switch ($data['action']) {
            case "associate":
                $this->associate($state);
                break;
            case "disassociate":
                $this->disassociate($state);
                break;
            default:
                throw new \RuntimeException("invalid action specified: " . $data['action']);
        }
    }

    public function associate(IosAppState $appState)
    {
        try {
            $this->vppLicenseService->associate($appState);
            $this->mdmAdmin->installApp($appState->getDevice(), $appState->getApp());
        } catch (VppAlreadyAssociatedException $e) {
            $this->logger->info("App is already assigned to user", array(
                "state"         => $appState->getId(),
                "user"          => $appState->getDevice()->getUser()->getId(),
                "userLogin"     => $appState->getDevice()->getUser()->getLogin(),
                "app"           => $appState->getApp()->getId(),
                "appIdentifier" => $appState->getApp()->getIdentifier(),
            ));
            $this->mdmAdmin->installApp($appState->getDevice(), $appState->getApp());
        } catch (VppException $e) {
            $this->logger->error("Unable to assign VPP license to user", array(
                "exception"     => $e,
                "state"         => $appState->getId(),
                "user"          => $appState->getDevice()->getUser()->getId(),
                "userLogin"     => $appState->getDevice()->getUser()->getLogin(),
                "app"           => $appState->getApp()->getId(),
                "appIdentifier" => $appState->getApp()->getIdentifier(),
            ));
        }
    }

    public function cleanup()
    {
    }

    public function disassociate(IosAppState $appState)
    {
        try {
            $this->vppLicenseService->disassociate($appState);
        } catch (VppException $e) {
            $this->logger->error("Unable to disassociate VPP license from user", array(
                "exception"     => $e,
                "state"         => $appState->getId(),
                "user"          => $appState->getDevice()->getUser()->getId(),
                "userLogin"     => $appState->getDevice()->getUser()->getLogin(),
                "app"           => $appState->getApp()->getId(),
                "appIdentifier" => $appState->getApp()->getIdentifier(),
            ));
        }
    }

    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }
}
